"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkCode = exports.build = void 0;
const child_process_1 = require("child_process");
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const perf_hooks_1 = require("perf_hooks");
const _1 = require(".");
const utils_1 = require("./utils");
let _builtWorkspaces = false, _builtBinaries = false, _ranCargoCheck = false;
/**
 * Build with Cross
 */
function build(options) {
    try {
        let outputName;
        let shouldCompile;
        let extra_args;
        // Build binary
        if (options.bin) {
            outputName = options.bin;
            if (_builtBinaries) {
                shouldCompile = false;
                extra_args = undefined;
            }
            else if (_1.Settings.BUILD_INDIVIDUALLY) {
                shouldCompile = true;
                extra_args = ['--bin', outputName];
            }
            else {
                _builtBinaries = true;
                shouldCompile = true;
                extra_args = ['--bins'];
            }
        }
        // Build package - i.e. workspace member
        else {
            outputName = options.package;
            if (_builtWorkspaces) {
                shouldCompile = false;
                extra_args = undefined;
            }
            else if (_1.Settings.BUILD_INDIVIDUALLY) {
                shouldCompile = true;
                extra_args = ['--package', outputName];
            }
            else {
                _builtWorkspaces = true;
                shouldCompile = true;
                extra_args = ['--workspace'];
            }
        }
        let targetReleaseDir = path.join(options.entry, 'target', options.target, 'release');
        const releaseDirExists = fs.existsSync(targetReleaseDir);
        if (shouldCompile) {
            // Base arguments for `cargo check` and `cross build`
            const buildArgs = [];
            let extraBuildArgs = options.extraBuildArgs || _1.Settings.EXTRA_BUILD_ARGS;
            let features = options.features || _1.Settings.FEATURES;
            if (extraBuildArgs) {
                buildArgs.push(...extraBuildArgs);
            }
            if (features) {
                buildArgs.push('--features', features.join(','));
            }
            // Set process environment (optional)
            let inputEnv = options.buildEnvironment ||
                _1.Settings.BUILD_ENVIRONMENT;
            const buildEnv = inputEnv
                ? {
                    ...process.env,
                    ...inputEnv,
                }
                : undefined;
            // Run `cargo check` on an initial time, if needed
            if (_1.Settings.RUN_CARGO_CHECK && !_ranCargoCheck) {
                _ranCargoCheck = true;
                checkCode(options, buildArgs, buildEnv);
            }
            if (releaseDirExists) {
                console.log(`🍺  Building Rust code...`);
            }
            else {
                // The `release` directory doesn't exist for the specified
                // target. This is most likely an initial run, so `cross` will
                // take much longer than usual to cross-compile the code.
                //
                // Print out an informative message that the `build` step is
                // expected to take longer than usual.
                console.log(`🍺  Building Rust code with \`cross\`. This may take a few minutes...`);
            }
            const args = [
                'build',
                '--release',
                '--target',
                options.target,
                ...buildArgs,
                ...extra_args,
            ];
            // Pass compile-time environment variables into `cross build`.
            // See <https://github.com/cross-rs/cross#passing-environment-variables-into-the-build-environment>.
            let createdCrossTomlFile = undefined;
            if (inputEnv) {
                // Create and use a temporary `Cross.toml`, if needed
                let filePath = pathToCrossToml(options.entry);
                if (!fs.existsSync(filePath)) {
                    createdCrossTomlFile = true;
                    const fileContents = `[build.env]\npassthrough = ${JSON.stringify(Object.keys(inputEnv))}`;
                    fs.writeFileSync(filePath, fileContents);
                }
            }
            const cross = (0, child_process_1.spawnSync)('cross', args, {
                cwd: options.entry,
                env: buildEnv,
            });
            if (createdCrossTomlFile) {
                (0, utils_1.deleteFile)(pathToCrossToml(options.entry));
            }
            if (cross.error) {
                throw cross.error;
            }
            if (cross.status !== 0) {
                throw new Error(cross.stderr.toString().trim());
            }
        }
        let from = path.join(targetReleaseDir, outputName);
        let to = path.join(options.outDir, 'bootstrap');
        fs.copyFileSync(from, to);
    }
    catch (err) {
        throw new Error(`Failed to build file at ${options.entry}: ${err}`);
    }
}
exports.build = build;
/**
 * Validate code with `cargo check`
 *
 * Note: this step is optional, and can be disabled with
 * `Settings.RUN_CARGO_CHECK` as needed.
 */
function checkCode(options, buildArgs, buildEnv) {
    let targetReleaseDir = path.join(options.entry, 'target', 'release');
    const releaseDirExists = fs.existsSync(targetReleaseDir);
    if (!releaseDirExists) {
        // The `release` directory doesn't exist for the specified
        // target. This is most likely an initial run, so `cargo` will
        // take much longer than usual to check the code.
        //
        // Print out an informative message that the `validate` step is
        // expected to take longer than usual.
        console.log(`🧪  Checking code with \`cargo\`. This may take a few minutes...`);
    }
    let start = perf_hooks_1.performance.now();
    const args = [
        'check',
        '--release',
        ...buildArgs,
        '--color',
        'always',
    ];
    const check = (0, child_process_1.spawnSync)('cargo', args, {
        cwd: options.entry,
        env: buildEnv,
    });
    if (check.error) {
        throw check.error;
    }
    if (check.status !== 0) {
        console.error(check.stderr.toString().trim());
        console.error(`💥  Run \`cargo check\` errored.`);
        process.exit(1);
        // Note: I don't want to raise an error here, as that will clutter the
        // output with the stack trace here. But maybe, there's a way to
        // suppress that?
        // throw new Error(check.stderr.toString().trim());
    }
    (0, utils_1.logTime)(start, `✅  Run \`cargo check\``);
}
exports.checkCode = checkCode;
function pathToCrossToml(entry) {
    return path.join(entry, 'Cross.toml');
}
